'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function __extends(d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var Coordinates = (function () {
    function Coordinates(col, row) {
        this.col = col;
        this.row = row;
    }
    Coordinates.from = function (col, row) {
        return new this(col, row);
    };
    Coordinates.fromIndex = function (index) {
        var col = index % 8;
        var row = Math.floor(index / 8);
        return new this(col, row);
    };
    Coordinates.prototype.buildPath = function (getNext) {
        var next = getNext(this);
        var result = [];
        while (next !== null) {
            result.push(next);
            next = getNext(next);
        }
        return result;
    };
    Object.defineProperty(Coordinates.prototype, "index", {
        get: function () {
            return this.row * 8 + this.col;
        },
        enumerable: true,
        configurable: true
    });
    Coordinates.prototype.up = function () {
        if (this.row <= 0) {
            return null;
        }
        return Coordinates.from(this.col, this.row - 1);
    };
    Coordinates.prototype.down = function () {
        if (this.row >= 7) {
            return null;
        }
        return Coordinates.from(this.col, this.row + 1);
    };
    Coordinates.prototype.right = function () {
        if (this.col >= 7) {
            return null;
        }
        return Coordinates.from(this.col + 1, this.row);
    };
    Coordinates.prototype.left = function () {
        if (this.col <= 0) {
            return null;
        }
        return Coordinates.from(this.col - 1, this.row);
    };
    Coordinates.prototype.upleft = function () {
        var step = this.up();
        return step && step.left();
    };
    Coordinates.prototype.upright = function () {
        var step = this.up();
        return step && step.right();
    };
    Coordinates.prototype.downleft = function () {
        var step = this.down();
        return step && step.left();
    };
    Coordinates.prototype.downright = function () {
        var step = this.down();
        return step && step.right();
    };
    Coordinates.prototype.upPath = function () {
        return this.buildPath(function (c) { return c.up(); });
    };
    Coordinates.prototype.downPath = function () {
        return this.buildPath(function (c) { return c.down(); });
    };
    Coordinates.prototype.leftPath = function () {
        return this.buildPath(function (c) { return c.left(); });
    };
    Coordinates.prototype.rightPath = function () {
        return this.buildPath(function (c) { return c.right(); });
    };
    Coordinates.prototype.upleftPath = function () {
        return this.buildPath(function (c) { return c.upleft(); });
    };
    Coordinates.prototype.uprightPath = function () {
        return this.buildPath(function (c) { return c.upright(); });
    };
    Coordinates.prototype.downleftPath = function () {
        return this.buildPath(function (c) { return c.downleft(); });
    };
    Coordinates.prototype.downrightPath = function () {
        return this.buildPath(function (c) { return c.downright(); });
    };
    return Coordinates;
}());

(function (Player) {
    Player[Player["None"] = 0] = "None";
    Player[Player["CPU"] = 1] = "CPU";
    Player[Player["Human"] = 2] = "Human";
})(exports.Player || (exports.Player = {}));

var extractWeight = function (_a) {
    var piece = _a.piece, coordinates = _a.coordinates;
    return piece.weight(coordinates);
};
var sum = function (acc, i) { return acc + i; };
var score = function (board, depth) {
    if (depth === void 0) { depth = 0; }
    var cpuScore = +board.fieldsByPlayer(exports.Player.CPU).map(extractWeight).reduce(sum, 0).toFixed(2);
    var humanScore = +board.fieldsByPlayer(exports.Player.Human).map(extractWeight).reduce(sum, 0).toFixed(2);
    return cpuScore - humanScore - depth;
};

var Move = (function () {
    function Move(from, to) {
        this.from = from;
        this.to = to;
    }
    Move.sortBy = function (player, board) {
        return function (m1, m2) {
            return player === exports.Player.CPU
                ? score(board.makeMove(m2)) - score(board.makeMove(m1))
                : score(board.makeMove(m1)) - score(board.makeMove(m2));
        };
    };
    return Move;
}());
var EvaluatedMove = (function (_super) {
    __extends(EvaluatedMove, _super);
    function EvaluatedMove(from, to, score$$1) {
        var _this = _super.call(this, from, to) || this;
        _this.score = score$$1;
        return _this;
    }
    EvaluatedMove.from = function (move, score$$1) {
        return new EvaluatedMove(move.from, move.to, score$$1);
    };
    EvaluatedMove.sortBy = function (player) {
        return function (em1, em2) {
            return player === exports.Player.CPU ? em2.score - em1.score : em1.score - em2.score;
        };
    };
    return EvaluatedMove;
}(Move));

var Field = (function () {
    function Field(_coordinates, _piece) {
        if (_piece === void 0) { _piece = null; }
        this._coordinates = _coordinates;
        this._piece = _piece;
    }
    Object.defineProperty(Field.prototype, "piece", {
        get: function () {
            return this._piece;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Field.prototype, "coordinates", {
        get: function () {
            return this._coordinates;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Field.prototype, "isEmpty", {
        get: function () {
            return this._piece == null;
        },
        enumerable: true,
        configurable: true
    });
    Field.prototype.clone = function (piece) {
        if (piece === void 0) { piece = null; }
        var _a = this, _b = _a._coordinates, col = _b.col, row = _b.row, _piece = _a._piece;
        return new Field(Coordinates.from(col, row), (_piece && _piece.clone()) || piece);
    };
    Field.prototype.toString = function () {
        var _a = this._coordinates, row = _a.row, col = _a.col;
        var piece = this.isEmpty ?
            (' ') : this._piece.toString();
        var cell = "(" + col + "," + row + ")";
        return "[" + piece + cell + "]";
    };
    Field.prototype.possibleMoves = function (board) {
        var _this = this;
        return this.isEmpty
            ? []
            : this._piece
                .possibleMoves(this._coordinates, board)
                .map(function (coordinate) { return new Move(_this._coordinates, coordinate); });
    };
    return Field;
}());

var Piece = (function () {
    function Piece(player) {
        this.player = player;
        this._weight = 0;
    }
    Piece.prototype.possibleMoves = function (coordinates, board) {
        return [];
    };
    Piece.prototype.weight = function (coordinates) {
        return this._weight;
    };
    Piece.prototype.toString = function () {
        return '';
    };
    Piece.prototype.clone = function () {
        throw new Error('not implemented in abstract class');
    };
    Piece.prototype.possiblePathMoves = function (path, board, includeStrikes) {
        if (includeStrikes === void 0) { includeStrikes = true; }
        var moves = [];
        for (var _i = 0, path_1 = path; _i < path_1.length; _i++) {
            var coord = path_1[_i];
            var field = board.at(coord);
            if (field.isEmpty) {
                moves.push(coord);
            }
            else if (field.piece.player !== this.player) {
                includeStrikes && moves.push(coord);
                break;
            }
            else
                break;
        }
        return moves;
    };
    return Piece;
}());

var Pawn = (function (_super) {
    __extends(Pawn, _super);
    function Pawn() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._weight = 1;
        return _this;
    }
    Pawn.prototype.toString = function () {
        return this.player == exports.Player.Human ? '♙' : '♟';
    };
    Pawn.prototype.weight = function (_a) {
        var row = _a.row;
        var coefficient = 0.01;
        var human = this.player === exports.Player.Human;
        return human ? this._weight - row * coefficient : this._weight - (7 - row) * coefficient;
    };
    Pawn.prototype.possibleMoves = function (coordinates, board) {
        var _this = this;
        var human = this.player === exports.Player.Human;
        var freeMoves = [];
        if (human) {
            var up = coordinates.up();
            if (up !== null && board.at(up).isEmpty) {
                freeMoves.push(up);
                if (coordinates.row === 6) {
                    var up2 = up.up();
                    if (board.at(up2).isEmpty) {
                        freeMoves.push(up2);
                    }
                }
            }
        }
        else {
            var down = coordinates.down();
            if (down !== null && board.at(down).isEmpty) {
                freeMoves.push(down);
                if (coordinates.row === 1) {
                    var down2 = down.down();
                    if (board.at(down2).isEmpty) {
                        freeMoves.push(down2);
                    }
                }
            }
        }
        var strikes = human
            ? [coordinates.upleft(), coordinates.upright()]
            : [coordinates.downleft(), coordinates.downright()];
        strikes = strikes.filter(function (coord) { return coord !== null && !board.at(coord).isEmpty && board.at(coord).piece.player !== _this.player; });
        return freeMoves.concat(strikes);
    };
    Pawn.prototype.clone = function () {
        return new Pawn(this.player);
    };
    return Pawn;
}(Piece));

var Rook = (function (_super) {
    __extends(Rook, _super);
    function Rook() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._weight = 2;
        return _this;
    }
    Rook.prototype.toString = function () {
        return this.player == exports.Player.Human ? '♖' : '♜';
    };
    Rook.prototype.clone = function () {
        return new Rook(this.player);
    };
    Rook.prototype.possibleMoves = function (coordinates, board) {
        return this.possiblePathMoves(coordinates.upPath(), board).concat(this.possiblePathMoves(coordinates.downPath(), board), this.possiblePathMoves(coordinates.leftPath(), board), this.possiblePathMoves(coordinates.rightPath(), board));
    };
    return Rook;
}(Piece));

var Knight = (function (_super) {
    __extends(Knight, _super);
    function Knight() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._weight = 3;
        return _this;
    }
    Knight.prototype.toString = function () {
        return this.player == exports.Player.Human ? '♘' : '♞';
    };
    Knight.prototype.clone = function () {
        return new Knight(this.player);
    };
    Knight.prototype.possibleMoves = function (coord, board) {
        var _this = this;
        var moves = [
            coord.up() && coord.up().upleft(),
            coord.up() && coord.up().upright(),
            coord.right() && coord.right().upright(),
            coord.right() && coord.right().downright(),
            coord.down() && coord.down().downright(),
            coord.down() && coord.down().downleft(),
            coord.left() && coord.left().downleft(),
            coord.left() && coord.left().upleft()
        ];
        return moves.filter(function (coord) {
            return coord !== null && (board.at(coord).isEmpty || board.at(coord).piece.player !== _this.player);
        });
    };
    return Knight;
}(Piece));

var Bishop = (function (_super) {
    __extends(Bishop, _super);
    function Bishop() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._weight = 2;
        return _this;
    }
    Bishop.prototype.toString = function () {
        return this.player == exports.Player.Human ? '♗' : '♝';
    };
    Bishop.prototype.clone = function () {
        return new Bishop(this.player);
    };
    Bishop.prototype.possibleMoves = function (coordinates, board) {
        return this.possiblePathMoves(coordinates.upleftPath(), board).concat(this.possiblePathMoves(coordinates.uprightPath(), board), this.possiblePathMoves(coordinates.downleftPath(), board), this.possiblePathMoves(coordinates.downrightPath(), board));
    };
    return Bishop;
}(Piece));

var Queen = (function (_super) {
    __extends(Queen, _super);
    function Queen() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._weight = 5;
        return _this;
    }
    Queen.prototype.toString = function () {
        return this.player == exports.Player.Human ? '♕' : '♛';
    };
    Queen.prototype.clone = function () {
        return new Queen(this.player);
    };
    Queen.prototype.possibleMoves = function (coordinates, board) {
        return this.possiblePathMoves(coordinates.upPath(), board).concat(this.possiblePathMoves(coordinates.uprightPath(), board), this.possiblePathMoves(coordinates.rightPath(), board), this.possiblePathMoves(coordinates.downrightPath(), board), this.possiblePathMoves(coordinates.downPath(), board), this.possiblePathMoves(coordinates.downleftPath(), board), this.possiblePathMoves(coordinates.leftPath(), board), this.possiblePathMoves(coordinates.upleftPath(), board));
    };
    return Queen;
}(Piece));

var King = (function (_super) {
    __extends(King, _super);
    function King() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._weight = 8;
        return _this;
    }
    King.prototype.toString = function () {
        return this.player == exports.Player.Human ? '♔' : '♚';
    };
    King.prototype.clone = function () {
        return new King(this.player);
    };
    King.prototype.possibleMoves = function (coordinates, board) {
        var _this = this;
        var moves = [
            coordinates.up(),
            coordinates.upright(),
            coordinates.right(),
            coordinates.downright(),
            coordinates.down(),
            coordinates.downleft(),
            coordinates.left(),
            coordinates.upleft()
        ];
        return moves.filter(function (coord) {
            return coord !== null && (board.at(coord).isEmpty || board.at(coord).piece.player !== _this.player);
        });
    };
    return King;
}(Piece));

var deserialize = function (str) {
    switch (str) {
        case '♟':
            return new Pawn(exports.Player.CPU);
        case '♜':
            return new Rook(exports.Player.CPU);
        case '♞':
            return new Knight(exports.Player.CPU);
        case '♝':
            return new Bishop(exports.Player.CPU);
        case '♛':
            return new Queen(exports.Player.CPU);
        case '♚':
            return new King(exports.Player.CPU);
        case '♙':
            return new Pawn(exports.Player.Human);
        case '♖':
            return new Rook(exports.Player.Human);
        case '♘':
            return new Knight(exports.Player.Human);
        case '♗':
            return new Bishop(exports.Player.Human);
        case '♕':
            return new Queen(exports.Player.Human);
        case '♔':
            return new King(exports.Player.Human);
        default:
            return null;
    }
};

// export const player = (s: string) => {
//   switch (s.toLowerCase()) {
//     case 'cpu': return Player.CPU;
//     case 'human': return Player.Human;
//     default: return Player.None;
//   }
// }
var opponent = function (player) {
    var CPU = exports.Player.CPU, Human = exports.Player.Human, None = exports.Player.None;
    switch (player) {
        case CPU:
            return Human;
        case Human:
            return CPU;
        default:
            return None;
    }
};
var Board = (function () {
    function Board(_fields) {
        this._fields = _fields;
    }
    Object.defineProperty(Board.prototype, "fields", {
        get: function () {
            return this._fields;
        },
        enumerable: true,
        configurable: true
    });
    Board.singleRowGenerator = function () {
        var WIDTH = 8;
        return Array.apply(null, { length: WIDTH }).map(Number.call, Number);
    };
    Board.coordGenerator = function (row) {
        return Board.singleRowGenerator().map(function (col) { return Coordinates.from(col, row); });
    };
    Board.piecesRow = function (coordinates, player) {
        var pieces = [
            new Rook(player),
            new Knight(player),
            new Bishop(player),
            new Queen(player),
            new King(player),
            new Bishop(player),
            new Knight(player),
            new Rook(player)
        ];
        return pieces.map(function (piece, index) { return new Field(coordinates[index], piece); });
    };
    Board.pawnRow = function (coordinates, player) {
        return coordinates.slice().map(function (coordinates) { return new Field(coordinates, new Pawn(player)); });
    };
    Board.emptyRow = function (coordinates) {
        return coordinates.slice().map(function (coordinates) { return new Field(coordinates); });
    };
    Board.emptyGame = function () {
        var fields = Board.emptyRow(Board.coordGenerator(0)).concat(Board.emptyRow(Board.coordGenerator(1)), Board.emptyRow(Board.coordGenerator(2)), Board.emptyRow(Board.coordGenerator(3)), Board.emptyRow(Board.coordGenerator(4)), Board.emptyRow(Board.coordGenerator(5)), Board.emptyRow(Board.coordGenerator(6)), Board.emptyRow(Board.coordGenerator(7)));
        return new Board(fields);
    };
    Board.fromJSON = function (json) {
        var fields = json.map(function (item, index) { return new Field(Coordinates.fromIndex(index), deserialize(item)); });
        return new Board(fields);
    };
    Board.newGame = function () {
        var fields = Board.piecesRow(Board.coordGenerator(0), exports.Player.CPU).concat(Board.pawnRow(Board.coordGenerator(1), exports.Player.CPU), Board.emptyRow(Board.coordGenerator(2)), Board.emptyRow(Board.coordGenerator(3)), Board.emptyRow(Board.coordGenerator(4)), Board.emptyRow(Board.coordGenerator(5)), Board.pawnRow(Board.coordGenerator(6), exports.Player.Human), Board.piecesRow(Board.coordGenerator(7), exports.Player.Human));
        return new Board(fields);
    };
    Board.prototype.at = function (coordinates) {
        return this.fields[coordinates.index];
    };
    Board.prototype.clone = function () {
        var fields = this.fields.map(function (field) { return field.clone(); });
        return new Board(fields);
    };
    Board.prototype.setAt = function (_a, piece) {
        var row = _a.row, col = _a.col, index = _a.index;
        var clone = this.clone();
        clone._fields[index] = new Field(Coordinates.from(col, row), piece);
        return clone;
    };
    Board.prototype.emptyAt = function (coordinates) {
        return this.setAt(coordinates, null);
    };
    Board.prototype.makeMove = function (_a) {
        var from = _a.from, to = _a.to;
        var _b = this.at(from), isEmpty = _b.isEmpty, piece = _b.piece;
        if (isEmpty)
            throw new Error('can`t make move from empty field');
        var clone = this.emptyAt(from).setAt(to, piece.clone());
        return clone;
    };
    Board.prototype.toString = function () {
        var str = '';
        this.fields.forEach(function (field, index) {
            var ending = (index + 1) % 8 === 0 ? '\n' : '';
            str += "" + field + ending;
        });
        return str;
    };
    Object.defineProperty(Board.prototype, "gameOver", {
        get: function () {
            return this.winner !== exports.Player.None;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Board.prototype, "winner", {
        get: function () {
            var _a = this.fieldsByPlayer(exports.Player.CPU).concat(this.fieldsByPlayer(exports.Player.Human)).filter(function (_a) {
                var piece = _a.piece;
                return piece instanceof King;
            })
                .map(function (_a) {
                var piece = _a.piece;
                return piece;
            }), king1 = _a[0], king2 = _a[1];
            return king1 && king2 ? exports.Player.None : king1.player;
        },
        enumerable: true,
        configurable: true
    });
    Board.prototype.fieldsByPlayer = function (player) {
        return this.fields.filter(function (field) { return !field.isEmpty && field.piece.player === player; });
    };
    Board.prototype.possibleMoves = function (player) {
        var _this = this;
        return this.fieldsByPlayer(player)
            .map(function (field) { return field.possibleMoves(_this); })
            .reduce(function (acc, moves) { return acc.concat(moves); }, []);
    };
    Board.prototype.toJSON = function () {
        return this.fields.map(function (_a) {
            var isEmpty = _a.isEmpty, piece = _a.piece;
            return (isEmpty ? null : "" + piece);
        });
    };
    return Board;
}());

var DEFAULT_DEPTH = 4;
var MAX_SCORE = Board.newGame().fieldsByPlayer(exports.Player.CPU).map(extractWeight).reduce(sum); //~34.9
var cpu = function (board, pruning, depth) {
    if (pruning === void 0) { pruning = false; }
    if (depth === void 0) { depth = DEFAULT_DEPTH; }
    var evaluatedMove = pruning ? alphabeta(board, depth) : minimax(board, depth);
    return board.makeMove(evaluatedMove);
};
var minimax = function (board, depth, player, move) {
    if (depth === void 0) { depth = DEFAULT_DEPTH; }
    if (player === void 0) { player = exports.Player.CPU; }
    var gameOver = board.gameOver, winner = board.winner;
    if (gameOver) {
        if (winner === exports.Player.CPU)
            return EvaluatedMove.from(move, MAX_SCORE - depth);
        if (winner === exports.Player.Human)
            return EvaluatedMove.from(move, depth - MAX_SCORE);
    }
    if (depth <= 0) {
        return EvaluatedMove.from(move, score(board, depth));
    }
    var opposingPlayer = opponent(player);
    var possibleMoves = board.possibleMoves(player);
    var evaluatedMoves = possibleMoves.map(function (move) {
        var newBoard = board.makeMove(move);
        var score$$1 = minimax(newBoard, depth - 1, opposingPlayer, move).score;
        return EvaluatedMove.from(move, score$$1);
    });
    var sortedMoves = evaluatedMoves.sort(EvaluatedMove.sortBy(player));
    var bestMove = sortedMoves[0];
    return bestMove ? bestMove : EvaluatedMove.from(move, score(board, depth));
};
var alphabeta = function (board, depth, player, move, alpha, beta) {
    if (depth === void 0) { depth = DEFAULT_DEPTH; }
    if (player === void 0) { player = exports.Player.CPU; }
    if (alpha === void 0) { alpha = -Infinity; }
    if (beta === void 0) { beta = +Infinity; }
    var gameOver = board.gameOver, winner = board.winner;
    if (gameOver) {
        if (winner === exports.Player.CPU)
            return EvaluatedMove.from(move, MAX_SCORE - depth);
        if (winner === exports.Player.Human)
            return EvaluatedMove.from(move, depth - MAX_SCORE);
    }
    if (depth <= 0) {
        return EvaluatedMove.from(move, score(board, depth));
    }
    var opposingPlayer = opponent(player);
    var possibleMoves = board.possibleMoves(player).sort(Move.sortBy(player, board));
    var maximizer = player === exports.Player.CPU;
    var evaluatedMoves = [];
    for (var _i = 0, possibleMoves_1 = possibleMoves; _i < possibleMoves_1.length; _i++) {
        var possibleMove = possibleMoves_1[_i];
        var newBoard = board.makeMove(possibleMove);
        var score_1 = alphabeta(newBoard, depth - 1, opposingPlayer, possibleMove, alpha, beta).score;
        evaluatedMoves.push(EvaluatedMove.from(possibleMove, score_1));
        if (maximizer) {
            if (score_1 > alpha) {
                alpha = score_1;
            }
        }
        else {
            if (score_1 < beta) {
                beta = score_1;
            }
        }
        if (alpha >= beta) {
            break;
        }
    }
    var sortedMoves = evaluatedMoves.sort(EvaluatedMove.sortBy(player));
    var bestMove = sortedMoves[0];
    return bestMove ? bestMove : EvaluatedMove.from(move, score(board, depth));
};

exports.Board = Board;
exports.opponent = opponent;
exports.Field = Field;
exports.Coordinates = Coordinates;
exports.Move = Move;
exports.EvaluatedMove = EvaluatedMove;
exports.cpu = cpu;
exports.minimax = minimax;
exports.alphabeta = alphabeta;
exports.Pawn = Pawn;
exports.Rook = Rook;
exports.Knight = Knight;
exports.Bishop = Bishop;
exports.Queen = Queen;
exports.King = King;
exports.Piece = Piece;
exports.extractWeight = extractWeight;
exports.score = score;
exports.sum = sum;
//# sourceMappingURL=index.js.map
